#include <autoconf.h>
#include <hermit.h>
#include <herrno.h>
#include <arch/system.h>
//#include <arch/iomux.h>
//#include <arch/clock.h>
#include <io.h>
#include <htypes.h>
#include <string.h>

extern int rma1_uart_post_setup(int port);

static int cur_console = 1;

struct rma1_uart {
	char *name;
	addr_t base;
};

struct rma1_uart uart[] = {
	{ .name = "none", },
	{ .name = "ttySC1", .base = SCIFA1_BASE_ADDR, },
	{ .name = "ttySC5", .base = SCIFA5_BASE_ADDR, },
};

static void rma1_setup_uart(int port)
{
	//rma1_per_clock_enable(15);
	//rma1_ipg_clock_enable(uart[port].clk);

	write16(uart[port].base + SCIFA_SCR, 0x0030);
	write16(uart[port].base + SCIFA_SMR, 0);

	/* reset FIFO */
	write16(uart[port].base + SCIFA_FCR, 0x0006);
	write16(uart[port].base + SCIFA_FCR, 0);

	/* set baudrate 115200 */
	write8(uart[port].base + SCIFA_BRR, 0x1a);
}

int change_console(char *console)
{
	int i;

	rma1_setup_uart(2);
	for (i=0; i<ARRAY_SIZE(uart); i++) {
		if (strncmp(console, uart[i].name, strlen(uart[i].name)) == 0
		    && rma1_uart_post_setup(i) == 0) {
			cur_console = i;
			if (uart[i].base)
				rma1_setup_uart(i);
			return 0;
		}
	}

	cur_console = 0;

	return -H_EINVAL;
}

int arch_getchar(unsigned long timeout)
{
	int ch;

	if (cur_console == 0)
		return -H_EINVAL;

	while (!(read16(uart[cur_console].base + SCIFA_SSR) &
		 (SSR_RDF | SSR_DR)));

	ch = (int)read8(uart[cur_console].base + SCIFA_FRDR);
	read16(uart[cur_console].base + SCIFA_SSR);
	write16(uart[cur_console].base + SCIFA_SSR, 0x00fc);

	return ch;
}

int arch_putchar(int c, unsigned long timeout)
{
	if (cur_console == 0)
		return -H_EINVAL;

	while (!((read16(uart[cur_console].base + SCIFA_FDR) >> 8) < FDR_FULL));
	write8(uart[cur_console].base + SCIFA_FTDR, c);

	write8(uart[2].base + SCIFA_FTDR, c);

	return 0;
}
